package SDON;

import java.util.ArrayList;

import SDON.sdonbuilder.sdonserialize.SDONSerializable;

/**
 * Object that represents a table that has been inserted into a shape.
 */
public class Table {
    /**
     * The number of rows in the table. This can be omitted for a default of 1
     * if there are columns defined. If neither the number of rows or columns
     * is defined, the Table object is ignored.
     */
    @SDONSerializable
    public int Rows = -1;
    
    /**
     * The number of columns in the table. This can be omitted for a default of
     * 1 if there are rows defined. If neither the number of  rows or columns
     * is defined, the Table object is ignored.
     */
    @SDONSerializable
    public int Columns = -1;
    
    /**
     * The width of the column in 1/100th inches.
     */
    @SDONSerializable
    public double ColumnWidth = -1.0;
    
    /**
     * Set the width of the column in 1/100th inches.
     * @param columnWidth - Column width
     * @return Table
     */
    public Table SetColumnWidth(double columnWidth) {
    	ColumnWidth = columnWidth;
    	
    	return this;
    }
    
    /**
     * The height of the row in 1/100th inches.
     */
    @SDONSerializable
    public double RowHeight = -1.0;
    
    /**
     * Set the height of the row in 1/100th inches.
     * @param rowHeight - Row height
     * @return Table
     */
    public Table SetRowHeight(double rowHeight) {
    	RowHeight = rowHeight;
    	
    	return this;
    }
    
    /**
     * Specific properties for individual cells.
     */
    @SDONSerializable
    public ArrayList<Cell> Cells = null;
    
    /**
     * Add a cell
     * @param row - Row of cell
     * @param column - Column of cell
     * @return Cell
     */
    public Cell AddCell(int row, int column) {
    	if (Cells == null)
    		Cells = new ArrayList<Cell>();
    	
    	Cell cell = new Cell();
    	cell.Row = row;
    	cell.Column = column;
    	
    	Cells.add(cell);
    	
    	return cell;
    }
    
    /**
     * Get a cell by its row and column position
     * @param row - Row of cell
     * @param column - Column of cell
     * @return Cell
     */
    public Cell GetCell(int row, int column) {
    	if (Cells == null)
    		return null;
    	
    	for (Cell cell : Cells) {
    		if ((cell.Row == row) && (cell.Column == column))
    			return cell;    		
    	}
    	
    	return null;
    }
    
    /**
     * Sets up a color scheme for alternating row colors in a table.
     */
    @SDONSerializable
    public TableAlternateRowsColors AlternateRows = null;
    
    /**
     * Sets up a color scheme for alternating row colors in a table.
     * @param color1 - First color of alternating row scheme
     * @param color2 - Second color of alternating row scheme
     * @return Table
     */
    public Table SetAlternateRows(String color1, String color2) {
    	AlternateRows = new TableAlternateRowsColors();
    	
    	AlternateRows.Color1 = color1;
    	AlternateRows.Color2 = color2;
    	
    	return this;
    }
    
    /**
     * List of elements that define a range of table cells to join into one cell.
     */
    @SDONSerializable
    public ArrayList<Join> Join = null;
    
    /**
     * Join a range of table cells to join into one cell horizontally
     * @param row - Starting row for the horizontal join
     * @param column - Starting column for the horizontal join
     * @param count - number of rows to join
     * @return Table
     */
    public Table JoinAcross(int row, int column, int count) {
    	return JoinCells(row, column, count, false);
    }
    
    /**
     * Join a range of table cells to join into one cell vertically
     * @param row - Starting row for the vertical join
     * @param column - Starting column for the vertical join
     * @param count - number of rows to join
     * @return Table
     */
    public Table JoinDown(int row, int column, int count) {
    	return JoinCells(row, column, count, true);
    }
    
    /**
     * Join multiple cells together into one cell
     * @param row
     * @param row - Starting row for the vertical join
     * @param column - Starting column for the vertical join
     * @param count - number of rows to join
     * @param down -  Join down (vertically) as opposed to accross (horizontally)
     * @return Table
     */
    private Table JoinCells(int row, int column, int count, Boolean down) {
    	if (Join == null)
    		Join = new ArrayList<Join>();
    	
    	Join join = new Join();
    	
    	join.Down = down ? 1 : 0;
    	
    	join.Row = row;
    	join.Column = column;
    	join.N = count;
    	
    	Join.add(join);
    	
    	return this;
    }
    
    /**
     * Special properties of individual columns.
     */
    @SDONSerializable
    public ArrayList<ColumnProperties> ColumnProperties = null;
    
    /**
     * Set the special properties of individual columns.
     * @param index - Index of column that the properties apply to
     * @return ColumnProperties
     */
    public ColumnProperties AddColumnProperties(int index) {
    	if (ColumnProperties == null) 
    		ColumnProperties = new ArrayList<ColumnProperties>();
    	
    	ColumnProperties columnProperties = new ColumnProperties();
    	columnProperties.Index = index;
    	
    	ColumnProperties.add(columnProperties);
    	
    	return columnProperties;
    }
    
    
    /**
     * Special properties of individual rows.
     */
    @SDONSerializable
    public ArrayList<RowProperties> RowProperties = null;
    
    /**
     * Set the special properties of individual rows.
     * @param index - index of row that the properties apply to
     * @return RowProperties
     */
    public RowProperties AddRowProperties(int index) {
    	if (RowProperties == null) 
    		RowProperties = new ArrayList<RowProperties>();
    	
    	RowProperties rowProperties = new RowProperties();
    	rowProperties.Index = index;
    	
    	RowProperties.add(rowProperties);
    	
    	return rowProperties;
    }
}